﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Data.Linq;
using LinqToSqlDemo.Properties;

namespace LinqToSqlDemo
{
	public partial class EmployeeSearchDialog : Form
	{
        private static readonly string _connStr = Settings.Default.NorthwindConnectionString;

		public EmployeeSearchDialog()
		{
			InitializeComponent();
		}

        private void PerformSearch()
        {
            NorthwindDataContext dc = new NorthwindDataContext(_connStr);

            // Start with the entire list of employees.  We will 
            // filter the list from here.
            IQueryable<Employee> list = dc.Employees;

            // Filter by first name (if entered)
            if (!string.IsNullOrEmpty(this.firstNameTextBox.Text))
            {
                list = from emp in list
                       where emp.FirstName.StartsWith(this.firstNameTextBox.Text)
                       select emp;
            }

            // Filter by last name (if entered)
            if (!string.IsNullOrEmpty(this.lastNameTextBox.Text))
            {
                list = from emp in list
                       where emp.LastName.StartsWith(this.lastNameTextBox.Text)
                       select emp;
            }



            // Get the data
            var result = from emp in list
                         select new EmployeeSummary
                         {
                             EmployeeID = emp.EmployeeID,
                             FirstName = emp.FirstName,
                             LastName = emp.LastName,
                             Title = emp.Title,
                             OrderCount = emp.Orders.Count,
                             Timestamp = emp.Timestamp
                         };

            result = result.OrderByDescending(es => es.OrderCount);

            // Put the data into the grid
            this.grid.DataSource = result.ToList();

            // Remove columns that we don't want to see in the grid
            this.grid.Columns.Remove("EmployeeID");
            this.grid.Columns.Remove("Timestamp");
        }

        private void searchButton_Click(object sender, EventArgs e)
		{
			PerformSearch();
		}

		private void addButton_Click(object sender, EventArgs e)
		{
            NorthwindDataContext dc = new NorthwindDataContext(_connStr);

			// Show the dialog so that the user can edit the new employee
			EditEmployeeDialog dlg = new EditEmployeeDialog();
			
			if (dlg.ShowDialog() == DialogResult.OK)
			{
				Employee emp = new Employee();

				// Get the data out of the dialog
				dlg.GetDataFromControls(emp);

				// Add the new Employee to the DataContext
				dc.Employees.InsertOnSubmit(emp);

				// Save the changes
				dc.SubmitChanges();

				// Perform the search again
				PerformSearch();
			}
		}

		private void editButton_Click(object sender, EventArgs e)
		{
			if (this.grid.SelectedRows.Count == 0)
			{
				MessageBox.Show("Don't know which row to edit!");
				return;
			}

            NorthwindDataContext dc = new NorthwindDataContext(_connStr);

			// Get the selected out of the grid
			EmployeeSummary employeeSummary = 
                (EmployeeSummary)this.grid.SelectedRows[0].DataBoundItem;

			// Show the dialog so that the user can edit the employee
			EditEmployeeDialog dlg = new EditEmployeeDialog();
			dlg.PopulateControls(employeeSummary);

			if (dlg.ShowDialog() == DialogResult.OK)
			{
				// Get the actual Employee object that we're going to edit
				Employee emp = (from employee in dc.Employees
								where employee.EmployeeID == employeeSummary.EmployeeID
								select employee).First();

				dlg.GetDataFromControls(emp);

				// Save the changes 
				dc.SubmitChanges();

				// Perform the search again
				PerformSearch();
			}
		}

		private void deleteButton_Click(object sender, EventArgs e)
		{
			if (this.grid.SelectedRows.Count == 0)
			{
				MessageBox.Show("Don't know which row to delete!");
				return;
			}

            NorthwindDataContext dc = new NorthwindDataContext(_connStr);

			EmployeeSummary emp = 
                (EmployeeSummary) this.grid.SelectedRows[0].DataBoundItem;
			
			// Create a simple Employee object with the necessary fields and 
			// attach it to the DataContext so that we can delete it.
			Employee actualEmployee = new Employee 
			{
				EmployeeID = emp.EmployeeID,
				Timestamp = emp.Timestamp
			};
			dc.Employees.Attach(actualEmployee);

			// Delete the employee from the DataContext
			dc.Employees.DeleteOnSubmit(actualEmployee);
			
			// Save the changes
			dc.SubmitChanges();

			// Perform the search again
			PerformSearch();
		}

        private void tenMostExpensiveProductsButton_Click(object sender, EventArgs e)
        {
            TenMostExpensiveProductsDialog dlg = new TenMostExpensiveProductsDialog();
            dlg.Show();
        }
	}
}
